

/* Generic definitions */




/* Assertions (useful to generate conditional code) */
/* Current type and class (and size, if applicable) */
/* Value methods */
/* Interfaces (keys) */
/* Interfaces (values) */
/* Abstract implementations (keys) */
/* Abstract implementations (values) */
/* Static containers (keys) */
/* Static containers (values) */
/* Implementations */
/* Synchronized wrappers */
/* Unmodifiable wrappers */
/* Other wrappers */
/* Methods (keys) */
/* Methods (values) */
/* Methods (keys/values) */
/* Methods that have special names depending on keys (but the special names depend on values) */
/* Equality */
/* Object/Reference-only definitions (keys) */
/* Primitive-type-only definitions (keys) */
/* Object/Reference-only definitions (values) */
/* Primitive-type-only definitions (values) */
/*		 
 * Copyright (C) 2002-2011 Sebastiano Vigna 
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */
package it.unimi.dsi.fastutil.doubles;
import it.unimi.dsi.fastutil.Hash;
import it.unimi.dsi.fastutil.HashCommon;
import it.unimi.dsi.fastutil.booleans.BooleanArrays;
import static it.unimi.dsi.fastutil.HashCommon.arraySize;
import static it.unimi.dsi.fastutil.HashCommon.maxFill;
import java.util.Map;
import java.util.NoSuchElementException;
import it.unimi.dsi.fastutil.doubles.DoubleCollection;
import it.unimi.dsi.fastutil.doubles.AbstractDoubleCollection;
import it.unimi.dsi.fastutil.doubles.DoubleIterator;
import java.util.Comparator;
import it.unimi.dsi.fastutil.doubles.DoubleListIterator;





import it.unimi.dsi.fastutil.objects.AbstractObjectSortedSet;
import it.unimi.dsi.fastutil.objects.ObjectListIterator;
import it.unimi.dsi.fastutil.objects.ObjectBidirectionalIterator;
import it.unimi.dsi.fastutil.objects.ObjectSortedSet;
/**  A type-specific linked hash map with with a fast, small-footprint implementation.
 *
 * <P>Instances of this class use a hash table to represent a map. The table is
 * enlarged as needed by doubling its size when new entries are created, but it is <em>never</em> made
 * smaller (even on a {@link #clear()}). A family of {@linkplain #trim() trimming
 * methods} lets you control the size of the table; this is particularly useful
 * if you reuse instances of this class.
 *
 * <P>Iterators created by this map will enumerate pairs in the same order in which they
 * have been added to the set (addition of pairs whose key is already present 
 * in the set doea not change the iteration order). Note that this order has nothing in common with the natural
 * order of the keys.
 *
 * <P>This class implements the interface of a sorted map, so to allow easy
 * access of the iteration order: for instance, you can get the first key
 * in iteration order with {@link #firstKey()} without having to create an
 * iterator; however, this class partially violates the {@link java.util.SortedMap}
 * contract because all submap methods throw an exception and {@link
 * #comparator()} returns always <code>null</code>.
 *
 * <P>The iterators provided by the views of this class using are type-specific
 * {@linkplain java.util.ListIterator list iterators}. However, creation of an
 * iterator using a starting point is going to be very expensive, as the chosen
 * starting point must be linearly searched for, unless it is {@link #lastKey()},
 * in which case the iterator is created in constant time.
 *
 * <P>Note that deletions in a linked map are very expensive, as they usually
 * require scanning the list until the element to be removed and the elements
 * shifted because of the removal are found.
 *
 * <p><strong>Warning:</strong> The implementation of this class has significantly
 * changed in <code>fastutil</code> 6.1.0. Please read the
 * comments about this issue in the section &ldquo;Faster Hash Tables&rdquo; of the <a href="../../../../../overview-summary.html">overview</a>.
 *
 * @see Hash
 * @see HashCommon
 */
public class Double2DoubleLinkedOpenHashMap extends AbstractDouble2DoubleSortedMap implements java.io.Serializable, Cloneable, Hash {
    public static final long serialVersionUID = 0L;
 private static final boolean ASSERTS = false;
 /** The array of keys. */
 protected transient double key[];
 /** The array of values. */
 protected transient double value[];
 /** The array telling whether a position is used. */
 protected transient boolean used[];
 /** The acceptable load factor. */
 protected final float f;
 /** The current table size. */
 protected transient int n;
 /** Threshold after which we rehash. It must be the table size times {@link #f}. */
 protected transient int maxFill;
 /** The mask for wrapping a position counter. */
 protected transient int mask;
 /** Number of entries in the set. */
 protected int size;
 /** Cached set of entries. */
 protected transient volatile FastSortedEntrySet entries;
 /** Cached set of keys. */
 protected transient volatile DoubleSortedSet keys;
 /** Cached collection of values. */
 protected transient volatile DoubleCollection values;
 /** The index of the first entry in iteration order. It is valid iff {@link #size} is nonzero; otherwise, it contains -1. */
 protected transient int first = -1;
 /** The index of the last entry in iteration order. It is valid iff {@link #size} is nonzero; otherwise, it contains -1. */
 protected transient int last = -1;
 /** For each entry, the next and the previous entry in iteration order
	exclusive-or'd together. It is valid only on {@link Hash#OCCUPIED}
	entries. The first and the last entry contain the actual successor and
	predecessor, respectively, exclusived-or'd with -1. */
 protected transient int link[];
 /** Creates a new hash map.
	 *
	 * <p>The actual table size will be the least power of two greater than <code>expected</code>/<code>f</code>.
	 *
	 * @param expected the expected number of elements in the hash set. 
	 * @param f the load factor.
	 */
 @SuppressWarnings("unchecked")
 public Double2DoubleLinkedOpenHashMap( final int expected, final float f ) {
  if ( f <= 0 || f > 1 ) throw new IllegalArgumentException( "Load factor must be greater than 0 and smaller than or equal to 1" );
  if ( expected < 0 ) throw new IllegalArgumentException( "The expected number of elements must be nonnegative" );
  this.f = f;
  n = arraySize( expected, f );
  mask = n - 1;
  maxFill = maxFill( n, f );
  key = new double[ n ];
  value = new double[ n ];
  used = new boolean[ n ];
  link = new int[ n ];
 }
 /** Creates a new hash map with {@link Hash#DEFAULT_LOAD_FACTOR} as load factor.
	 *
	 * @param expected the expected number of elements in the hash map.
	 */
 public Double2DoubleLinkedOpenHashMap( final int expected ) {
  this( expected, DEFAULT_LOAD_FACTOR );
 }
 /** Creates a new hash map with initial expected {@link Hash#DEFAULT_INITIAL_SIZE} entries
	 * and {@link Hash#DEFAULT_LOAD_FACTOR} as load factor.
	 */
 public Double2DoubleLinkedOpenHashMap() {
  this( DEFAULT_INITIAL_SIZE, DEFAULT_LOAD_FACTOR );
 }
 /** Creates a new hash map copying a given one.
	 *
	 * @param m a {@link Map} to be copied into the new hash map. 
	 * @param f the load factor.
	 */
 public Double2DoubleLinkedOpenHashMap( final Map<? extends Double, ? extends Double> m, final float f ) {
  this( m.size(), f );
  putAll( m );
 }
 /** Creates a new hash map with {@link Hash#DEFAULT_LOAD_FACTOR} as load factor copying a given one.
	 *
	 * @param m a {@link Map} to be copied into the new hash map. 
	 */
 public Double2DoubleLinkedOpenHashMap( final Map<? extends Double, ? extends Double> m ) {
  this( m, DEFAULT_LOAD_FACTOR );
 }
 /** Creates a new hash map copying a given type-specific one.
	 *
	 * @param m a type-specific map to be copied into the new hash map. 
	 * @param f the load factor.
	 */
 public Double2DoubleLinkedOpenHashMap( final Double2DoubleMap m, final float f ) {
  this( m.size(), f );
  putAll( m );
 }
 /** Creates a new hash map with {@link Hash#DEFAULT_LOAD_FACTOR} as load factor copying a given type-specific one.
	 *
	 * @param m a type-specific map to be copied into the new hash map. 
	 */
 public Double2DoubleLinkedOpenHashMap( final Double2DoubleMap m ) {
  this( m, DEFAULT_LOAD_FACTOR );
 }
 /** Creates a new hash map using the elements of two parallel arrays.
	 *
	 * @param k the array of keys of the new hash map.
	 * @param v the array of corresponding values in the new hash map.
	 * @param f the load factor.
	 * @throws IllegalArgumentException if <code>k</code> and <code>v</code> have different lengths.
	 */
 public Double2DoubleLinkedOpenHashMap( final double[] k, final double v[], final float f ) {
  this( k.length, f );
  if ( k.length != v.length ) throw new IllegalArgumentException( "The key array and the value array have different lengths (" + k.length + " and " + v.length + ")" );
  for( int i = 0; i < k.length; i++ ) this.put( k[ i ], v[ i ] );
 }
 /** Creates a new hash map with {@link Hash#DEFAULT_LOAD_FACTOR} as load factor using the elements of two parallel arrays.
	 *
	 * @param k the array of keys of the new hash map.
	 * @param v the array of corresponding values in the new hash map.
	 * @throws IllegalArgumentException if <code>k</code> and <code>v</code> have different lengths.
	 */
 public Double2DoubleLinkedOpenHashMap( final double[] k, final double v[] ) {
  this( k, v, DEFAULT_LOAD_FACTOR );
 }
 /*
	 * The following methods implements some basic building blocks used by
	 * all accessors. They are (and should be maintained) identical to those used in OpenHashSet.drv.
	 */
 public double put(final double k, final double v) {
  // The starting point.
  int pos = (int)it.unimi.dsi.fastutil.HashCommon.murmurHash3(Double.doubleToRawLongBits(k)) & mask;
  // There's always an unused entry.
  while( used[ pos ] ) {
   if ( ( (k) == (key[ pos ]) ) ) {
    final double oldValue = value[ pos ];
    value[ pos ] = v;
    return oldValue;
   }
   pos = ( pos + 1 ) & mask;
  }
  used[ pos ] = true;
  key[ pos ] = k;
  value[ pos ] = v;
  if ( size == 0 ) {
   first = last = pos;
   link[ pos ] = 0;
  }
  else {
   link[ last ] ^= pos ^ -1;
   link[ pos ] = last ^ -1;
   last = pos;
  }
  if ( ++size >= maxFill ) rehash( arraySize( size + 1, f ) );
  if ( ASSERTS ) checkTable();
  return defRetValue;
 }
 public Double put( final Double ok, final Double ov ) {
  final double v = ((ov).doubleValue());
  final double k = ((ok).doubleValue());
  // The starting point.
  int pos = (int)it.unimi.dsi.fastutil.HashCommon.murmurHash3(Double.doubleToRawLongBits(k)) & mask;
  // There's always an unused entry.
  while( used[ pos ] ) {
   if ( ( (k) == (key[ pos ]) ) ) {
    final Double oldValue = (Double.valueOf(value[ pos ]));
    value[ pos ] = v;
    return oldValue;
   }
   pos = ( pos + 1 ) & mask;
  }
  used[ pos ] = true;
  key[ pos ] = k;
  value[ pos ] = v;
  if ( size == 0 ) {
   first = last = pos;
   link[ pos ] = 0;
  }
  else {
   link[ last ] ^= pos ^ -1;
   link[ pos ] = last ^ -1;
   last = pos;
  }
  if ( ++size >= maxFill ) rehash( arraySize( size + 1, f ) );
  if ( ASSERTS ) checkTable();
  return (null);
 }
 /** Adds an increment to value currently associated with a key.
	 *
	 * <P>Note that this method respects the {@linkplain #defaultReturnValue() default return value} semantics: when
	 * called with a key that does not currently appears in the map, the key
	 * will be associated with the default return value plus
	 * the given increment.
	 *
	 * @param k the key.
	 * @param incr the increment.
	 * @return the old value, or the {@linkplain #defaultReturnValue() default return value} if no value was present for the given key.
	 */
 public double add(final double k, final double incr) {
  // The starting point.
  int pos = (int)it.unimi.dsi.fastutil.HashCommon.murmurHash3(Double.doubleToRawLongBits(k)) & mask;
  // There's always an unused entry.
  while( used[ pos ] ) {
   if ( ( (k) == (key[ pos ]) ) ) {
    final double oldValue = value[ pos ];
    value[ pos ] += incr;
    return oldValue;
   }
   pos = ( pos + 1 ) & mask;
  }
  used[ pos ] = true;
  key[ pos ] = k;
  value[ pos ] = (double)(defRetValue + incr);
  if ( size == 0 ) {
   first = last = pos;
   link[ pos ] = 0;
  }
  else {
   link[ last ] ^= pos ^ -1;
   link[ pos ] = last ^ -1;
   last = pos;
  }
  if ( ++size >= maxFill ) rehash( arraySize( size + 1, f ) );
  if ( ASSERTS ) checkTable();
  return defRetValue;
 }
 /** Shifts left entries with the specified hash code, starting at the specified position,
	 * and empties the resulting free entry.
	 *
	 * @param pos a starting position.
	 */
 protected final void shiftKeys( int pos ) {
  // Shift entries with the same hash.
  int last, slot;
  for(;;) {
   pos = ( ( last = pos ) + 1 ) & mask;
   while( used[ pos ] ) {
    slot = (int)it.unimi.dsi.fastutil.HashCommon.murmurHash3(Double.doubleToRawLongBits(key[ pos ])) & mask;
    if ( last <= pos ? last >= slot || slot > pos : last >= slot && slot > pos ) break;
    pos = ( pos + 1 ) & mask;
   }
   if ( ! used[ pos ] ) break;
   key[ last ] = key[ pos ];
   value[ last ] = value[ pos ];
   fixPointers( pos, last );
  }
  used[ last ] = false;
 }
 @SuppressWarnings("unchecked")
 public double remove( final double k ) {
  // The starting point.
  int pos = (int)it.unimi.dsi.fastutil.HashCommon.murmurHash3(Double.doubleToRawLongBits( k)) & mask;
  // There's always an unused entry.
  while( used[ pos ] ) {
   if ( ( ( k) == (key[ pos ]) ) ) {
    size--;
    fixPointers( pos );
    final double v = value[ pos ];
    shiftKeys( pos );
    return v;
   }
   pos = ( pos + 1 ) & mask;
  }
  return defRetValue;
 }
 @SuppressWarnings("unchecked")
 public Double remove( final Object ok ) {
  final double k = ((((Double)(ok)).doubleValue()));
  // The starting point.
  int pos = (int)it.unimi.dsi.fastutil.HashCommon.murmurHash3(Double.doubleToRawLongBits(k)) & mask;
  // There's always an unused entry.
  while( used[ pos ] ) {
   if ( ( (k) == (key[ pos ]) ) ) {
    size--;
    fixPointers( pos );
    final double v = value[ pos ];
    shiftKeys( pos );
    return (Double.valueOf(v));
   }
   pos = ( pos + 1 ) & mask;
  }
  return (null);
 }
 public Double get( final Double ok ) {
  final double k = ((ok).doubleValue());
  // The starting point.
  int pos = (int)it.unimi.dsi.fastutil.HashCommon.murmurHash3(Double.doubleToRawLongBits( k)) & mask;
  // There's always an unused entry.
  while( used[ pos ] ) {
   if ( ( ( k) == (key[ pos ]) ) ) return (Double.valueOf(value[ pos ]));
   pos = ( pos + 1 ) & mask;
  }
  return (null);
 }
 @SuppressWarnings("unchecked")
 public double get( final double k ) {
  // The starting point.
  int pos = (int)it.unimi.dsi.fastutil.HashCommon.murmurHash3(Double.doubleToRawLongBits( k)) & mask;
  // There's always an unused entry.
  while( used[ pos ] ) {
   if ( ( ( k) == (key[ pos ]) ) ) return value[ pos ];
   pos = ( pos + 1 ) & mask;
  }
  return defRetValue;
 }
 @SuppressWarnings("unchecked")
 public boolean containsKey( final double k ) {
  // The starting point.
  int pos = (int)it.unimi.dsi.fastutil.HashCommon.murmurHash3(Double.doubleToRawLongBits( k)) & mask;
  // There's always an unused entry.
  while( used[ pos ] ) {
   if ( ( ( k) == (key[ pos ]) ) ) return true;
   pos = ( pos + 1 ) & mask;
  }
  return false;
 }
 public boolean containsValue( final double v ) {
  final double value[] = this.value;
  final boolean used[] = this.used;
  for( int i = n; i-- != 0; ) if ( used[ i ] && ( (value[ i ]) == (v) ) ) return true;
  return false;
 }
 /* Removes all elements from this map.
	 *
	 * <P>To increase object reuse, this method does not change the table size.
	 * If you want to reduce the table size, you must use {@link #trim()}.
	 *
	 */
 public void clear() {
  if ( size == 0 ) return;
  size = 0;
  BooleanArrays.fill( used, false );
  // We null all object entries so that the garbage collector can do its work.
  first = last = -1;
 }
 public int size() {
  return size;
 }
 public boolean isEmpty() {
  return size == 0;
 }
 /** A no-op for backward compatibility.
	 * 
	 * @param growthFactor unused.
	 * @deprecated Since <code>fastutil</code> 6.1.0, hash tables are doubled when they are too full.
	 */
 @Deprecated
 public void growthFactor( int growthFactor ) {}
 /** Gets the growth factor (2).
	 *
	 * @return the growth factor of this set, which is fixed (2).
	 * @see #growthFactor(int)
	 * @deprecated Since <code>fastutil</code> 6.1.0, hash tables are doubled when they are too full.
	 */
 @Deprecated
 public int growthFactor() {
  return 16;
 }
 /** The entry class for a hash map does not record key and value, but
	 * rather the position in the hash table of the corresponding entry. This
	 * is necessary so that calls to {@link java.util.Map.Entry#setValue(Object)} are reflected in
	 * the map */
 private final class MapEntry implements Double2DoubleMap.Entry , Map.Entry<Double, Double> {
  private int index;
  MapEntry( final int index ) {
   this.index = index;
  }
  public Double getKey() {
   return (Double.valueOf(key[ index ]));
  }
  public double getDoubleKey() {
      return key[ index ];
  }
  public Double getValue() {
   return (Double.valueOf(value[ index ]));
  }
  public double getDoubleValue() {
   return value[ index ];
  }
  public double setValue( final double v ) {
   final double oldValue = value[ index ];
   value[ index ] = v;
   return oldValue;
  }
  public Double setValue( final Double v ) {
   return (Double.valueOf(setValue( ((v).doubleValue()) )));
  }
  @SuppressWarnings("unchecked")
  public boolean equals( final Object o ) {
   if (!(o instanceof Map.Entry)) return false;
   Map.Entry<Double, Double> e = (Map.Entry<Double, Double>)o;
   return ( (key[ index ]) == (((e.getKey()).doubleValue())) ) && ( (value[ index ]) == (((e.getValue()).doubleValue())) );
  }
  public int hashCode() {
   return it.unimi.dsi.fastutil.HashCommon.double2int(key[ index ]) ^ it.unimi.dsi.fastutil.HashCommon.double2int(value[ index ]);
  }
  public String toString() {
   return key[ index ] + "=>" + value[ index ];
  }
 }
 /** Modifies the {@link #link} vector so that the given entry is removed.
	 *
	 * <P>If the given entry is the first or the last one, this method will complete
	 * in constant time; otherwise, it will have to search for the given entry.
	 *
	 * @param i the index of an entry. 
	 */
 protected void fixPointers( final int i ) {
  if ( size == 0 ) {
   first = last = -1;
   return;
  }
  if ( first == i ) {
   first = link[ i ] ^ -1;
   link[ first ] ^= i ^ -1;
   return;
  }
  if ( last == i ) {
   last = link[ i ] ^ -1;
   link[ last ] ^= i ^ -1;
   return;
  }
  int j = first, prev = -1, next;
  while( ( next = link[ j ] ^ prev ) != i ) {
   prev = j;
   j = next;
  }
  link[ j ] ^= link[ i ] ^ i ^ j;
  link[ link[ i ] ^ j ] ^= i ^ j;
 }
 /** Modifies the {@link #link} vector for a shift from s to d.
	 *
	 * <P>If the given entry is the first or the last one, this method will complete
	 * in constant time; otherwise, it will have to search for the given entry.
	 *
	 * @param s the source position.
	 * @param d the destination position.
	 */
 protected void fixPointers( int s, int d ) {
  if ( size == 1 ) {
   first = last = d;
   link[ d ] = 0;
   return;
  }
  if ( first == s ) {
   first = d;
   link[ link[ s ] ^ -1 ] ^= s ^ d;
   link[ d ] = link[ s ];
   return;
  }
  if ( last == s ) {
   last = d;
   link[ link[ s ] ^ -1 ] ^= s ^ d;
   link[ d ] = link[ s ];
   return;
  }
  int j = first, prev = -1, next;
  while( ( next = link[ j ] ^ prev ) != s ) {
   prev = j;
   j = next;
  }
  link[ j ] ^= s ^ d;
  link[ link[ s ] ^ j ] ^= s ^ d;
  link[ d ] = link[ s ];
 }
 /** Returns the first key of this map in iteration order.
	 *
	 * @return the first key in iteration order.
	 */
 public double firstDoubleKey() {
  if ( size == 0 ) throw new NoSuchElementException();
  return key[ first ];
 }
 /** Returns the last key of this map in iteration order.
	 *
	 * @return the last key in iteration order.
	 */
 public double lastDoubleKey() {
  if ( size == 0 ) throw new NoSuchElementException();
  return key[ last ];
 }
 public DoubleComparator comparator() { return null; }
 public Double2DoubleSortedMap tailMap( double from ) { throw new UnsupportedOperationException(); }
 public Double2DoubleSortedMap headMap( double to ) { throw new UnsupportedOperationException(); }
 public Double2DoubleSortedMap subMap( double from, double to ) { throw new UnsupportedOperationException(); }
 /** A list iterator over a linked map.
	 *
	 * <P>This class provides a list iterator over a linked hash map. The empty constructor runs in 
	 * constant time. The one-argument constructor needs to search for the given key, but it is 
	 * optimized for the case of {@link java.util.SortedMap#lastKey()}, in which case runs in constant time, too.
	 */
 private class MapIterator {
  /** The entry that will be returned by the next call to {@link java.util.ListIterator#previous()} (or <code>null</code> if no previous entry exists). */
  int prev = -1;
  /** The entry that will be returned by the next call to {@link java.util.ListIterator#next()} (or <code>null</code> if no next entry exists). */
  int next = -1;
  /** The last entry that was returned (or -1 if we did not iterate or used {@link java.util.Iterator#remove()}). */
  int curr = -1;
  /** The current index (in the sense of a {@link java.util.ListIterator}). Note that this value is not meaningful when this iterator has been created using the nonempty constructor.*/
  int index = 0;
  private MapIterator() {
   next = first;
  }
  private MapIterator( final double from ) {
   if ( ( (key[ last ]) == (from) ) ) {
    prev = last;
    index = size;
   }
   else {
    if ( ! Double2DoubleLinkedOpenHashMap.this.containsKey( from ) ) throw new IllegalArgumentException( "The key " + from + " does not belong to this set." );
    next = first;
    int e;
    do e = nextEntry(); while( ! ( (key[ e ]) == (from) ) );
    curr = -1;
   }
  }
  public boolean hasNext() { return next != -1; }
  public boolean hasPrevious() { return prev != -1; }
  public int nextIndex() {
   return index;
  }
  public int previousIndex() {
   return index - 1;
  }
  public int nextEntry() {
   if ( ! hasNext() ) return size();
   curr = next;
   next = link[ curr ] ^ prev;
   prev = curr;
   index++;
   return curr;
  }
  public int previousEntry() {
   if ( ! hasPrevious() ) return -1;
   curr = prev;
   prev = link[ curr ] ^ next;
   next = curr;
   index--;
   return curr;
  }
  @SuppressWarnings("unchecked")
  public void remove() {
   if ( curr == -1 ) throw new IllegalStateException();
   if ( curr == prev ) {
    /* If the last operation was a next(), we are removing an entry that preceeds
				   the current index, and thus we must decrement it. */
    index--;
    prev = link[ curr ] ^ next;
   }
   else next = link[ curr ] ^ prev; // curr == next
   size--;
   /* Now we manually fix the pointers. Because of our knowledge of next
			   and prev, this is going to be faster than calling fixPointers(). */
   if ( prev == -1 ) first = next;
   else link[ prev ] ^= curr ^ next;
   if ( next == -1 ) last = prev;
   else link[ next ] ^= curr ^ prev;
   int last, slot, pos = curr;
   // We have to horribly duplicated the shiftKeys() code because we need to update next/prev.			
   for(;;) {
    pos = ( ( last = pos ) + 1 ) & mask;
    while( used[ pos ] ) {
     slot = (int)it.unimi.dsi.fastutil.HashCommon.murmurHash3(Double.doubleToRawLongBits(key[ pos ])) & mask;
     if ( last <= pos ? last >= slot || slot > pos : last >= slot && slot > pos ) break;
     pos = ( pos + 1 ) & mask;
    }
    if ( ! used[ pos ] ) break;
    key[ last ] = key[ pos ];
    value[ last ] = value[ pos ];
    if ( next == pos ) next = last;
    if ( prev == pos ) prev = last;
    fixPointers( pos, last );
   }
   used[ last ] = false;
   curr = -1;
  }
  public int skip( final int n ) {
   int i = n;
   while( i-- != 0 && hasNext() ) nextEntry();
   return n - i - 1;
  }
  public int back( final int n ) {
   int i = n;
   while( i-- != 0 && hasPrevious() ) previousEntry();
   return n - i - 1;
  }
 }
 private class EntryIterator extends MapIterator implements ObjectListIterator<Double2DoubleMap.Entry > {
  public EntryIterator() {}
  public EntryIterator( double from ) {
   super( from );
  }
  public MapEntry next() {
   return new MapEntry( nextEntry() );
  }
  public MapEntry previous() {
   return new MapEntry( previousEntry() );
  }
  public void set( Double2DoubleMap.Entry ok ) { throw new UnsupportedOperationException(); }
  public void add( Double2DoubleMap.Entry ok ) { throw new UnsupportedOperationException(); }
 }
 private class FastEntryIterator extends MapIterator implements ObjectListIterator<Double2DoubleMap.Entry > {
  final BasicEntry entry = new BasicEntry ( (0), (0) );
  public FastEntryIterator() {}
  public FastEntryIterator( double from ) {
   super( from );
  }
  public BasicEntry next() {
   final int e = nextEntry();
   entry.key = key[ e ];
   entry.value = value[ e ];
   return entry;
  }
  public BasicEntry previous() {
   final int e = previousEntry();
   entry.key = key[ e ];
   entry.value = value[ e ];
   return entry;
  }
  public void set( Double2DoubleMap.Entry ok ) { throw new UnsupportedOperationException(); }
  public void add( Double2DoubleMap.Entry ok ) { throw new UnsupportedOperationException(); }
 }
 private final class MapEntrySet extends AbstractObjectSortedSet<Double2DoubleMap.Entry > implements FastSortedEntrySet {
  public ObjectBidirectionalIterator<Double2DoubleMap.Entry > iterator() {
   return new EntryIterator();
  }
  public Comparator<? super Double2DoubleMap.Entry > comparator() { return null; }
  public ObjectSortedSet<Double2DoubleMap.Entry > subSet( Double2DoubleMap.Entry fromElement, Double2DoubleMap.Entry toElement) { throw new UnsupportedOperationException(); }
  public ObjectSortedSet<Double2DoubleMap.Entry > headSet( Double2DoubleMap.Entry toElement ) { throw new UnsupportedOperationException(); }
  public ObjectSortedSet<Double2DoubleMap.Entry > tailSet( Double2DoubleMap.Entry fromElement ) { throw new UnsupportedOperationException(); }
  public Double2DoubleMap.Entry first() {
   if ( size == 0 ) throw new NoSuchElementException();
   return new MapEntry( Double2DoubleLinkedOpenHashMap.this.first );
  }
  public Double2DoubleMap.Entry last() {
   if ( size == 0 ) throw new NoSuchElementException();
   return new MapEntry( Double2DoubleLinkedOpenHashMap.this.last );
  }
  @SuppressWarnings("unchecked")
  public boolean contains( final Object o ) {
   if ( !( o instanceof Map.Entry ) ) return false;
   final Map.Entry<Double, Double> e = (Map.Entry<Double, Double>)o;
   final double k = ((e.getKey()).doubleValue());
   // The starting point.
   int pos = (int)it.unimi.dsi.fastutil.HashCommon.murmurHash3(Double.doubleToRawLongBits(k)) & mask;
   // There's always an unused entry.
   while( used[ pos ] ) {
    if ( ( (k) == (key[ pos ]) ) ) return ( (value[ pos ]) == (((e.getValue()).doubleValue())) );
    pos = ( pos + 1 ) & mask;
   }
   return false;
  }
  @SuppressWarnings("unchecked")
  public boolean remove( final Object o ) {
   if ( !( o instanceof Map.Entry ) ) return false;
   final Map.Entry<Double, Double> e = (Map.Entry<Double, Double>)o;
   final double k = ((e.getKey()).doubleValue());
   // The starting point.
   int pos = (int)it.unimi.dsi.fastutil.HashCommon.murmurHash3(Double.doubleToRawLongBits(k)) & mask;
   // There's always an unused entry.
   while( used[ pos ] ) {
    if ( ( (k) == (key[ pos ]) ) ) {
     Double2DoubleLinkedOpenHashMap.this.remove( e.getKey() );
     return true;
    }
    pos = ( pos + 1 ) & mask;
   }
   return false;
  }
  public int size() {
   return size;
  }
  public void clear() {
   Double2DoubleLinkedOpenHashMap.this.clear();
  }
  public ObjectBidirectionalIterator<Double2DoubleMap.Entry > iterator( final Double2DoubleMap.Entry from ) {
   return new EntryIterator( ((from.getKey()).doubleValue()) );
  }
  public ObjectBidirectionalIterator<Double2DoubleMap.Entry > fastIterator() {
   return new FastEntryIterator();
  }
  public ObjectBidirectionalIterator<Double2DoubleMap.Entry > fastIterator( final Double2DoubleMap.Entry from ) {
   return new FastEntryIterator( ((from.getKey()).doubleValue()) );
  }
 }
 public FastSortedEntrySet double2DoubleEntrySet() {
  if ( entries == null ) entries = new MapEntrySet();
  return entries;
 }
 /** An iterator on keys.
	 *
	 * <P>We simply override the {@link java.util.ListIterator#next()}/{@link java.util.ListIterator#previous()} methods
	 * (and possibly their type-specific counterparts) so that they return keys
	 * instead of entries.
	 */
 private final class KeyIterator extends MapIterator implements DoubleListIterator {
  public KeyIterator( final double k ) { super( k ); }
  public double previousDouble() { return key[ previousEntry() ]; }
  public void set( double k ) { throw new UnsupportedOperationException(); }
  public void add( double k ) { throw new UnsupportedOperationException(); }
  public Double previous() { return (Double.valueOf(key[ previousEntry() ])); }
  public void set( Double ok ) { throw new UnsupportedOperationException(); }
  public void add( Double ok ) { throw new UnsupportedOperationException(); }
  public KeyIterator() { super(); }
  public double nextDouble() { return key[ nextEntry() ]; }
  public Double next() { return (Double.valueOf(key[ nextEntry() ])); }
 }
 private final class KeySet extends AbstractDoubleSortedSet {
  public DoubleBidirectionalIterator iterator( final double from ) {
   return new KeyIterator( from );
  }
  public DoubleBidirectionalIterator iterator() {
   return new KeyIterator();
  }
  public int size() {
   return size;
  }
  public boolean contains( double k ) {
   return containsKey( k );
  }
  public boolean remove( double k ) {
   final int oldSize = size;
   Double2DoubleLinkedOpenHashMap.this.remove( k );
   return size != oldSize;
  }
  public void clear() {
   Double2DoubleLinkedOpenHashMap.this.clear();
  }
  public double firstDouble() {
   if ( size == 0 ) throw new NoSuchElementException();
   return key[ first ];
  }
  public double lastDouble() {
   if ( size == 0 ) throw new NoSuchElementException();
   return key[ last ];
  }
  public DoubleComparator comparator() { return null; }
  final public DoubleSortedSet tailSet( double from ) { throw new UnsupportedOperationException(); }
  final public DoubleSortedSet headSet( double to ) { throw new UnsupportedOperationException(); }
  final public DoubleSortedSet subSet( double from, double to ) { throw new UnsupportedOperationException(); }
 }
 public DoubleSortedSet keySet() {
  if ( keys == null ) keys = new KeySet();
  return keys;
 }
 /** An iterator on values.
	 *
	 * <P>We simply override the {@link java.util.ListIterator#next()}/{@link java.util.ListIterator#previous()} methods
	 * (and possibly their type-specific counterparts) so that they return values
	 * instead of entries.
	 */
 private final class ValueIterator extends MapIterator implements DoubleListIterator {
  public double previousDouble() { return value[ previousEntry() ]; }
  public Double previous() { return (Double.valueOf(value[ previousEntry() ])); }
  public void set( Double ok ) { throw new UnsupportedOperationException(); }
  public void add( Double ok ) { throw new UnsupportedOperationException(); }
  public void set( double v ) { throw new UnsupportedOperationException(); }
  public void add( double v ) { throw new UnsupportedOperationException(); }
  public ValueIterator() { super(); }
  public double nextDouble() { return value[ nextEntry() ]; }
  public Double next() { return (Double.valueOf(value[ nextEntry() ])); }
 }
 public DoubleCollection values() {
  if ( values == null ) values = new AbstractDoubleCollection () {
    public DoubleIterator iterator() {
     return new ValueIterator();
    }
    public int size() {
     return size;
    }
    public boolean contains( double v ) {
     return containsValue( v );
    }
    public void clear() {
     Double2DoubleLinkedOpenHashMap.this.clear();
    }
   };
  return values;
 }
 /** A no-op for backward compatibility. The kind of tables implemented by
	 * this class never need rehashing.
	 *
	 * <P>If you need to reduce the table size to fit exactly
	 * this set, use {@link #trim()}.
	 *
	 * @return true.
	 * @see #trim()
	 * @deprecated A no-op.
	 */
 @Deprecated
 public boolean rehash() {
  return true;
 }
 /** Rehashes the map, making the table as small as possible.
	 * 
	 * <P>This method rehashes the table to the smallest size satisfying the
	 * load factor. It can be used when the set will not be changed anymore, so
	 * to optimize access speed and size.
	 *
	 * <P>If the table size is already the minimum possible, this method
	 * does nothing. 
	 *
	 * @return true if there was enough memory to trim the map.
	 * @see #trim(int)
	 */
 public boolean trim() {
  final int l = arraySize( size, f );
  if ( l >= n ) return true;
  try {
   rehash( l );
  }
  catch(OutOfMemoryError cantDoIt) { return false; }
  return true;
 }
 /** Rehashes this map if the table is too large.
	 * 
	 * <P>Let <var>N</var> be the smallest table size that can hold
	 * <code>max(n,{@link #size()})</code> entries, still satisfying the load factor. If the current
	 * table size is smaller than or equal to <var>N</var>, this method does
	 * nothing. Otherwise, it rehashes this map in a table of size
	 * <var>N</var>.
	 *
	 * <P>This method is useful when reusing maps.  {@linkplain #clear() Clearing a
	 * map} leaves the table size untouched. If you are reusing a map
	 * many times, you can call this method with a typical
	 * size to avoid keeping around a very large table just
	 * because of a few large transient maps.
	 *
	 * @param n the threshold for the trimming.
	 * @return true if there was enough memory to trim the map.
	 * @see #trim()
	 */
 public boolean trim( final int n ) {
  final int l = HashCommon.nextPowerOfTwo( (int)Math.ceil( n / f ) );
  if ( this.n <= l ) return true;
  try {
   rehash( l );
  }
  catch( OutOfMemoryError cantDoIt ) { return false; }
  return true;
 }
 /** Resizes the map.
	 *
	 * <P>This method implements the basic rehashing strategy, and may be
	 * overriden by subclasses implementing different rehashing strategies (e.g.,
	 * disk-based rehashing). However, you should not override this method
	 * unless you understand the internal workings of this class.
	 *
	 * @param newN the new size
	 */
 @SuppressWarnings("unchecked")
 protected void rehash( final int newN ) {
  int i = first, prev = -1, newPrev = -1, t, pos;
  double k;
  final double key[] = this.key;
  final double value[] = this.value;
  final int newMask = newN - 1;
  final double newKey[] = new double[ newN ];
  final double newValue[] = new double[newN];
  final boolean newUsed[] = new boolean[ newN ];
  final int link[] = this.link;
  final int newLink[] = new int[ newN ];
  first = -1;
  for( int j = size; j-- != 0; ) {
   k = key[ i ];
   pos = (int)it.unimi.dsi.fastutil.HashCommon.murmurHash3(Double.doubleToRawLongBits(k)) & newMask;
   while ( newUsed[ pos ] ) pos = ( pos + 1 ) & newMask;
   newUsed[ pos ] = true;
   newKey[ pos ] = k;
   newValue[ pos ] = value[ i ];
   if ( prev != -1 ) {
    newLink[ newPrev ] ^= pos;
    newLink[ pos ] = newPrev;
    newPrev = pos;
   }
   else {
    newPrev = first = pos;
    newLink[ pos ] = -1;
   }
   t = i;
   i = link[ i ] ^ prev;
   prev = t;
  }
  n = newN;
  mask = newMask;
  maxFill = maxFill( n, f );
  this.key = newKey;
  this.value = newValue;
  this.used = newUsed;
  this.link = newLink;
  this.last = newPrev;
  if ( newPrev != -1 ) newLink[ newPrev ] ^= -1;
 }
 /** Returns a deep copy of this map. 
	 *
	 * <P>This method performs a deep copy of this hash map; the data stored in the
	 * map, however, is not cloned. Note that this makes a difference only for object keys.
	 *
	 *  @return a deep copy of this map.
	 */
 @SuppressWarnings("unchecked")
 public Double2DoubleLinkedOpenHashMap clone() {
  Double2DoubleLinkedOpenHashMap c;
  try {
   c = (Double2DoubleLinkedOpenHashMap )super.clone();
  }
  catch(CloneNotSupportedException cantHappen) {
   throw new InternalError();
  }
  c.keys = null;
  c.values = null;
  c.entries = null;
  c.key = key.clone();
  c.value = value.clone();
  c.used = used.clone();
  c.link = link.clone();
  return c;
 }
 /** Returns a hash code for this map.
	 *
	 * This method overrides the generic method provided by the superclass. 
	 * Since <code>equals()</code> is not overriden, it is important
	 * that the value returned by this method is the same value as
	 * the one returned by the overriden method.
	 *
	 * @return a hash code for this map.
	 */
 public int hashCode() {
  int h = 0;
  for( int j = size, i = 0, t = 0; j-- != 0; ) {
   while( ! used[ i ] ) i++;
    t = it.unimi.dsi.fastutil.HashCommon.double2int(key[ i ]);
    t ^= it.unimi.dsi.fastutil.HashCommon.double2int(value[ i ]);
   h += t;
   i++;
  }
  return h;
 }
 private void writeObject(java.io.ObjectOutputStream s) throws java.io.IOException {
  final double key[] = this.key;
  final double value[] = this.value;
  final MapIterator i = new MapIterator();
  s.defaultWriteObject();
  for( int j = size, e; j-- != 0; ) {
   e = i.nextEntry();
   s.writeDouble( key[ e ] );
   s.writeDouble( value[ e ] );
  }
 }
 @SuppressWarnings("unchecked")
 private void readObject(java.io.ObjectInputStream s) throws java.io.IOException, ClassNotFoundException {
  s.defaultReadObject();
  n = arraySize( size, f );
  maxFill = maxFill( n, f );
  mask = n - 1;
  final double key[] = this.key = new double[ n ];
  final double value[] = this.value = new double[ n ];
  final boolean used[] = this.used = new boolean[ n ];
  final int link[] = this.link = new int[ n ];
  int prev = -1;
  first = last = -1;
  double k;
  double v;
  for( int i = size, pos = 0; i-- != 0; ) {
   k = s.readDouble();
   v = s.readDouble();
   pos = (int)it.unimi.dsi.fastutil.HashCommon.murmurHash3(Double.doubleToRawLongBits(k)) & mask;
   while ( used[ pos ] ) pos = ( pos + 1 ) & mask;
   used[ pos ] = true;
   key[ pos ] = k;
   value[ pos ] = v;
   if ( first != -1 ) {
    link[ prev ] ^= pos;
    link[ pos ] = prev;
    prev = pos;
   }
   else {
    prev = first = pos;
    link[ pos ] = -1;
   }
  }
  last = prev;
  if ( prev != -1 ) link[ prev ] ^= -1;
  if ( ASSERTS ) checkTable();
 }
 private void checkTable() {}
}
